@gipfl-color-low-sat-blue: #dae3e6;
// map Icinga vars, to avoid warnings in other files
@gipfl-color-main: @icinga-blue;
@gipfl-color-text: @text-color;
@gipfl-color-gray-light: @gray-light;
@gipfl-bg-element: #f2f1f0;
@gipfl-bg-element: @low-sat-blue;
@gipfl-icon-font: 'ifont';
