form.gipfl-quicksearch {
    display: block;
    input.search {
        width: 8em;
        min-width: unset;
        border: none;
        background-color: inherit;
        padding-left: 2em;
        margin-left: 1.5em;
        font-size: 0.75em;
        font-weight: normal;
        &:focus {
            width: 16em;
            border: none;
        }
    }
}
